#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import sys
import wanalyzer
#import wcomparison
import wccd
import wplanner
import wvisual
import koprdial
import datavar
import settings
import functools
import koprhand 
from PyQt5 import QtCore, QtGui, QtWidgets

mitm = [ 'File', 'Configuration', 'Database', 'Tools', 'Help' ]
sitm = [ 'Location', 'Telescope', 'Eyepiece', 'Camera' ]

# read config file
def doRead(fname):
    itemlist = []
    f = open('./config-files/'+fname+'.dat', 'r')
    for line in f.readlines():
      item = line.replace('\n','').replace('\r','').split(',')
      if len(item) > 1:
        itemlist.append(item)
    if fname == 'telescope': #Naked Eye hook
        itemlist.append( ['Naked Eye', 'E', '1', '0'] )   
    f.close()
    if fname == 'camera': #added settings patch
        f = open('./config-files/'+fname+'.dat', 'w')
        for item in itemlist:
          if len(item) == 8:
            item.append('')
          f.write(','.join(item)+'\n')
    f.close()    
    return itemlist

# Load data
def DataCheck():
    missing = koprhand.filecheck()
    koprhand.CometUpdate()
    return missing

def ItemCheck(observer, items, missing):
    message = []
    if missing == 'all':
      message = ['Fill all your settings first']
    else:
      if 'ICQxx' in observer or 'first name' in observer or 'second name' in observer:
        message.append('Fill your ICQ code and name first')
      if not items['location']:
        message.append('Input your observing location first')
      if not items['telescope'] or (['Naked Eye', 'E', '1', '0'] in items['telescope'] and len(items['telescope']) == 1):
        message.append('Input your observing telescope first')
      if not items['camera'] and not items['eyepiece']:
        message.append('Input your observing eyepiece or camera first')
    return message

def Initialize():
    settings = koprhand.SettingsRead()
    obscode, fname, lname = settings[0]
    defaults = settings[1]
    if len(defaults) == 11:
      defaults.append('0')
      defaults.append('0')

    observer, items = [ obscode, fname, lname ], {}
    for item in [ 'location', 'telescope', 'eyepiece', 'camera' ]:
      items[item] = doRead(item)
            
    return observer, defaults, items

# ************* APPLICATION MAIN WINDOW ************
class MainWindow(QtWidgets.QMainWindow):
    def __init__(self, parent=None):
        QtWidgets.QWidget.__init__(self, parent)
        self.missing = DataCheck()
        self.observer, self.defaults, self.items = Initialize()
        self.settings = [ self.observer, self.defaults, [], [], [], [] ]
        self.message = ItemCheck(self.observer, self.items, self.missing)

        if self.message:
          self.SetupFirst()
          while self.message:
            self.SetupFirst()

        #Actions
        #Toolbar actions
        self.Visual = QtWidgets.QAction(QtGui.QIcon('./icons/telescope.png'), 'Visual', self)
        self.Visual.triggered.connect(self.showVisual)
        self.CCD = QtWidgets.QAction(QtGui.QIcon('./icons/camera.png'), 'CCD', self)
        self.CCD.triggered.connect(self.showCCD)
        self.OPlanner = QtWidgets.QAction(QtGui.QIcon('./icons/obsplanner.png'), 'Obs Planner', self)
        self.OPlanner.triggered.connect(self.showOPlanner)
        self.Analyzer = QtWidgets.QAction(QtGui.QIcon('./icons/analysis.png'), 'Analyzer', self)
        self.Analyzer.triggered.connect(self.showAnalyzer)
        self.Comparison = QtWidgets.QAction(QtGui.QIcon('./icons/comet.png'), 'Comet comparison', self)
        self.Comparison.triggered.connect(self.showComparison)

        #Menubar actions
        self.Settings = QtWidgets.QAction(QtGui.QIcon('./icons/settings.png'), 'Settings', self)
        self.Settings.triggered.connect(self.showSettings)
        self.AboutUs = QtWidgets.QAction(QtGui.QIcon('./icons/comet.png') , 'About Us', self)
        self.AboutUs.triggered.connect(self.showAboutus)
        self.AddComet = QtWidgets.QAction(QtGui.QIcon('./icons/comet.png') , 'Add Comet', self)
        self.AddComet.triggered.connect(self.AddCometfunc)
#        self.ImgWiz = QtWidgets.QAction(QtGui.QIcon('./icons/camera.png') , 'DLSR Image tool', self)
#        self.ImgWiz.triggered.connect(self.ImageWizard)

        self.exitAction = QtWidgets.QAction(QtGui.QIcon('exit.png'), '&Exit', self)        
        self.exitAction.setShortcut('Ctrl+Q')
        self.exitAction.setStatusTip('Exit application')
        self.exitAction.triggered.connect(QtWidgets.qApp.quit)

        #Menu
        self.menubar = QtWidgets.QMenuBar()
        self.menu, self.submenu, self.smenu_action = [], [], []
        self.SetMenus()
        self.setMenuBar(self.menubar)

        #Toolbars
        self.toolbar = self.addToolBar('Visual')
        self.toolbar.addAction(self.Visual)
        self.toolbar = self.addToolBar('CCD')
        self.toolbar.addAction(self.CCD)
        self.toolbar = self.addToolBar('Obs Planner')
        self.toolbar.addAction(self.OPlanner)
        self.toolbar = self.addToolBar('Analyzer')
        self.toolbar.addAction(self.Analyzer)
        #self.toolbar = self.addToolBar('Comet comparison')
        #self.toolbar.addAction(self.Comparison)
        
        #geometry, statusbar
        if self.settings[2]: 
          self.statusBar().showMessage('Location: {:s} '.format( self.settings[2][0] ))
        else:
          self.statusBar().showMessage('')
        self.setGeometry(100, 100, 880, 620)
        self.setWindowTitle(datavar.Version()+' - {:s} {:s} ({:s})'.format( self.settings[0][1], self.settings[0][2], self.settings[0][0])) 
    
    # Update menu bar items on change
    def UpdateMenus(self):
        self.observer, self.defaults, self.items = Initialize()
        self.settings = [ self.observer, self.defaults, [], [], [], [] ]
        self.smenu_action = []
        for i in range(len(sitm)):
          self.submenu[i].clear() 
          self.smenu_action.append([])
          for index, item in enumerate(self.items[sitm[i].lower()]):
            self.smenu_action[i].append(QtWidgets.QAction(item[0], self.submenu[i], checkable=True))
            self.smenu_action[i][index].triggered.connect(functools.partial(self.SelControl, i, index))
            self.submenu[i].addAction(self.smenu_action[i][index])
          if self.items[sitm[i].lower()]:
            self.smenu_action[i][0].setChecked(1)
            self.settings[2+i] = self.items[sitm[i].lower()][0] 

    # Set menu bar
    def SetMenus(self):
        for i in range(len(mitm)):
          self.menu.append(QtWidgets.QMenu(mitm[i], self))

        #Menu file
        #self.menu[0].addAction(self.ImgWiz)
        self.menu[0].addAction(self.exitAction)
        self.menu[2].addAction(self.AddComet)
#        self.menu[3].addAction(self.ImgWiz)
        self.menu[4].addAction(self.AboutUs)

        #Menu settings
        self.menu[1].addAction(self.Settings)
        for i in range(len(sitm)):
          self.submenu.append(QtWidgets.QMenu(sitm[i], self))
          self.smenu_action.append([])
          for index, item in enumerate(self.items[sitm[i].lower()]):
            self.smenu_action[i].append(QtWidgets.QAction(item[0], self.submenu[i], checkable=True))
            self.smenu_action[i][index].triggered.connect(functools.partial(self.SelControl, i, index))
            self.submenu[i].addAction(self.smenu_action[i][index])

          self.menu[1].addMenu(self.submenu[i])

          if self.items[sitm[i].lower()]:
            self.smenu_action[i][0].setChecked(1)
            self.settings[2+i] = self.items[sitm[i].lower()][0]
          else:
            self.settings[2+i] = []

        for i in range(len(self.menu)):
          self.menubar.addMenu(self.menu[i])
        koprhand.SettingsUpdate(self.settings)        

    # Settings dialog open
    def showSettings(self):
        self.showSetWidget()
        self.UpdateMenus()#update menu if change

    def showSetWidget(self):
        self.items['telescope'].remove(['Naked Eye', 'E', '1', '0']) #Naked Eye hook
        tabdialog = settings.TabDialog(self.observer, self.defaults, self.items)
        tabdialog.exec_()
        observer, defaults = tabdialog.observer, tabdialog.defaults
        if observer != self.observer or defaults != self.defaults:
          self.observer, self.defaults = observer, defaults

          f = open('./config-files/settings.dat', 'r')
          content = f.read()
          content = content.split('\n')
          f.close()

          content[0] = ';'.join(observer)
          content[1] = ';'.join(defaults)

          f = open('./config-files/settings.dat', 'w')
          f.write( '\n'.join(content) )
          f.close()

          self.settings[0] = self.observer
          if self.settings[2]:
            self.statusBar().showMessage('Location: {:s} '.format( self.settings[2][0] ))
          else:
            self.statusBar().showMessage('')
          self.setWindowTitle(datavar.Version()+' - {:s} {:s} ({:s})'.format( self.settings[0][1], self.settings[0][2], self.settings[0][0]))

    def onResize(self, event):
        self.actualw.w.resize(event.size().width(), event.size().height())

    # Add visual observation
    def showVisual(self):
        self.actualw = wvisual.ScrollVisual(self) #passing settings to widget
        self.actualw.resizeEvent = self.onResize
        self.StatBarUpdate('ScrollVisual')
        self.setCentralWidget(self.actualw)

    # Add CCD measure
    def showCCD(self):
        self.actualw = wccd.ScrollCCD(self) #passing settings to widget
        self.actualw.resizeEvent = self.onResize
        self.StatBarUpdate('ScrollCCD')
        self.setCentralWidget(self.actualw)
        
    # Open observer planner
    def showOPlanner(self):
        self.actualw = wplanner.ScrollPlanner(self) #passing settings to widget
        self.actualw.resizeEvent = self.onResize
        self.StatBarUpdate('ScrollPlanner')
        self.setCentralWidget(self.actualw)
        
    # Open analyzer
    def showAnalyzer(self):
        self.actualw = wanalyzer.ScrollAnalyzer(self) #passing settings to widget
        self.actualw.resizeEvent = self.onResize
        self.StatBarUpdate('ScrollAnalyzer')
        self.setCentralWidget(self.actualw)

    # Open comparison
    def showComparison(self):
        self.actualw = wcomparison.ScrollComparison(self) #passing settings to widget
        self.actualw.resizeEvent = self.onResize
        self.StatBarUpdate('ScrollComparison')
        self.setCentralWidget(self.actualw)

    # About us section
    def showAboutus(self):
        koprdial.InfoDialog( 0, 'About '+datavar.Version(), datavar.Version()+\
                                                             ' is free software for comet observers and researchers\n \nAuthor: Jakub Cerny \nSociety for Interplannetary matter, Czech Republic\n\nE-#mail: kaos@kommet.cz\nWeb: ' )
        print(' is free software for comet observers and researchers\n \nAuthor: Jakub Cerny \nSociety for Interplannetary matter, Czech Republic\n\nE-mail: kaos@kommet.cz\nWeb:')
                                                             
    # Add custom comet
    def AddCometfunc(self):
        koprdial.AddComet()

    # Add custom comet
    def ImageWizard(self):
        koprdial.ImgWiz()

    # Status bar update
    def SelControl(self, cat, index):
        for item in self.smenu_action[cat]:
          item.setChecked(0)
        self.smenu_action[cat][index].setChecked(1)
        self.settings[2+cat] = self.items[sitm[cat].lower()][index]
        
        try:
          actual = type(self.actualw).__name__
          self.StatBarUpdate(actual)
        except:
          self.StatBarUpdate('')

        koprhand.SettingsUpdate(self.settings)        

        try: #passing settings to widget
          self.actualw.w.testvar = self.settings
        except: 
          print('failed to passing settings to widget')
          pass
     
    def StatBarUpdate(self, option):
        if self.settings[2]: 
            if option == 'ScrollVisual':# setting status bar for visual obs
              if self.settings[3][0] == 'Naked Eye' and self.settings[4]:
                 self.statusBar().showMessage('Location: {:s}, Naked Eye'.format( self.settings[2][0] ))
              elif not self.settings[4]:
                 self.message = ['Input eyepiece first']
                 self.SetupFirst()
                 self.message = []
              else: 
                if self.settings[3][3]:
                  focl = '(f/{:d})'.format( int( float(self.settings[3][3]) / float(self.settings[3][2])) )
                  zoom = int( round(float(self.settings[3][3]) / float(self.settings[4][1]),0 ))
                  fov = '{:5.2f}\u00b0'.format( round( float(self.settings[4][2]) / (float(self.settings[3][3])/float(self.settings[4][1])), 2 ) )
                
                elif len(self.settings[3]) == 6:
                  if self.settings[3][4] and self.settings[3][5]:
                    focl = ''
                    zoom = int( self.settings[3][4] )
                    fov = '{:5.2f}\u00b0'.format( float(self.settings[3][5]) )

                self.statusBar().showMessage('Location: {:s}, Aperture: {:s}mm {:s}, Zoom: {:d}x, FOV: {:s}'.format( self.settings[2][0], self.settings[3][2], focl, zoom, fov ))

            elif option == 'ScrollCCD': # setting status bar for ccd obs
              if self.settings[3][0] == 'Naked Eye':
                 self.statusBar().showMessage('Location: {:s}, You cannot provide CCD measures with Naked Eye'.format( self.settings[2][0] ))
              else: 
                if self.settings[3][3] and self.settings[5]:
                  tfocl, xres, yres, xpix, ypix = float(self.settings[3][3]), float(self.settings[5][4]), float(self.settings[5][5]), float(self.settings[5][6]), float(self.settings[5][7])
                  focl = '(f/{:d})'.format( int( tfocl / float(self.settings[3][2]) ) )
                  pxsize =  '{:4.1f}"x{:4.1f}"'.format( round(xpix/tfocl*206.3,1), round(ypix/tfocl*206.3,1) ) 
                  fov = '{:5.2f}\u00b0x{:5.2f}\u00b0'.format( round(xres*(xpix/tfocl*206.3)/3600.,2), round(yres*(ypix/tfocl*206.3)/3600.,2) ) 
                  self.statusBar().showMessage('Location: {:s}, Aperture: {:s}mm {:s}, Camera: {:s} (Pixelsize: {:s}, FOV: {:s})'.format( self.settings[2][0], self.settings[3][2], focl, self.settings[5][0], pxsize, fov ))
                elif not self.settings[5]:
                  self.message = ['Input camera first']
                  self.SetupFirst()
                  self.message = []
                elif len(self.settings[3]) == 6:
                  if self.settings[3][4] and self.settings[3][5]:
                    self.statusBar().showMessage('Location: {:s}, You cannot provide CCD measures with Telescope with Fixed Zoom'.format( self.settings[2][0] ))
              
            elif option == '': # no vidget is active
              self.statusBar().showMessage('Location: {:s} '.format( self.settings[2][0] ))  
              
        else: # missing location
            self.statusBar().showMessage('')
    def SetupFirst(self):
        text = ''
        for item in self.message:
          text += item+'\n'
        koprdial.InfoDialog(2, 'Configuration incomplete', text)
        self.showSetWidget()
        self.observer, self.defaults, self.items = Initialize()
        self.message = ItemCheck(self.observer, self.items, '')
        self.settings = [ self.observer, self.defaults, [], [], [], [] ]
        
if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    myapp = MainWindow()
    myapp.show()
    sys.exit(app.exec_())
