#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

from math import *
import numpy as np
import koprdial
import alb3

#translating to matrix of photometric center
def ImgToMatrix(x,y,dx,dy):
  return (x - dx), (y - dy)

#scaling result to pixel range
def DeltaY( yr , ym ):
  if ym < 0:
    ym = -( ym + 1.)

  dy = yr - ym
  if dy < 0:
    return 0.
  elif dy > 1:
    return 1.
  else:
    return dy

#integrating surface part of pixel to photom
def Integrace( xm, ym, r ):
  s, s1, s2 = 0., 0., 0.
  for i in range(11):
    xr = xm + i/10.    
    if abs(xr) <= r:
      yr = alb3.pytha(xr, r)
      #yr = ( r*r - xr*xr )**(1./2.)
      s1 = 0.1 * DeltaY( yr , ym )
      if i > 0 and abs(xr-0.1) <= r:
        s += (s1 + s2)/2.
      s2 = s1     
  return s

def Pythagoras(a,b):
  return (a*a + b*b)**(1./2.)

#copysign with zero
def cszero(orig):
  if orig == 0.:
    return 0.
  else:
    return copysign(0.5,orig)

#build photometric matrix of image
def PhotomMatrix( x0, y0 ,r ,shape ):
    field = np.zeros( shape, dtype=np.float32)
    #build search radius
    ranges = [ int(-r-1) , int(+r+2) ]
    #center pixel
    xc, yc = int(x0), int(y0) 
    #start
    for x in range( xc + ranges[0], xc + ranges[1] ):
      for y in range( yc + ranges[0], yc + ranges[1] ):
        xm, ym = ImgToMatrix( x, y ,x0 ,y0 )
        xb, yb = xm + cszero(xm), ym + cszero(ym)
        c = alb3.pythc(xb, yb)
        if c <= r:
          field[x,y] = 1.
        else:
          field[x,y] = Integrace( xm-0.5, ym-0.5, r)

    return field, [xc + ranges[0], xc + ranges[1], yc + ranges[0], yc + ranges[1]]

def Photometry( image_data, x0, y0, r, sky, limit ):
    error = 0
    phmat, rg = PhotomMatrix( y0, x0, r, image_data.shape )
    datacount = np.sum( (image_data[ rg[0]:rg[1], rg[2]:rg[3] ] - sky) * phmat[ rg[0]:rg[1], rg[2]:rg[3] ] )
    if limit in ( image_data[ rg[0]:rg[1], rg[2]:rg[3] ] * phmat[ rg[0]:rg[1], rg[2]:rg[3] ]):
      error = 1

    return 40.0-2.5*log10(datacount/sky), error    

#  ******** Averaging circles filter ********
def AverCircFlt( phmat, rg, image_data, sky, defaults ):
    #print '******** Averaging circles filter ********'
    #get difference circle
    field = phmat[ rg[0]:rg[1], rg[2]:rg[3] ]
    #intersection, sky deduced, weights applied
    result = (image_data[ rg[0]:rg[1], rg[2]:rg[3] ] - sky) * field
    origcnt = np.sum(result) 
    #print 'original count: ', np.sum(result) 
    #masking zeros
    result = np.ma.masked_where( result==0 , result)
    field = np.ma.masked_where( field==0 , field)
    #getting back original values without zeros
    result = result / field

    #calculating weighted average
    mavera = np.ma.average( result, weights = field )
    #print 'weighted average=', maver

    #calculating median
    maver = np.ma.median( result )
    #print 'median, average =', maver, mavera
    #masking bright pixels
    result = np.ma.masked_where( result > maver*( float(defaults[7])/100. + 1. ) , result)
    maver = np.ma.average( result, weights = field )
    #get count
    newcount = np.sum(field)*maver 
    #print 'ACM filtered count', newcount
    if newcount > origcnt: 
      return origcnt
    else:
      return newcount

def PhotometryACF( image_data, x0, y0, r, sky, limit, defaults ):
    afrho = {}
    error = 0
    i = int(defaults[4])
    
    #init value    
    phmat0, rg = PhotomMatrix( y0, x0, i, image_data.shape )
    datacount = np.sum( (image_data[ rg[0]:rg[1], rg[2]:rg[3] ] - sky) * phmat0[ rg[0]:rg[1], rg[2]:rg[3] ] )
    if limit in ( image_data[ rg[0]:rg[1], rg[2]:rg[3] ] * phmat0[ rg[0]:rg[1], rg[2]:rg[3] ]):
      error = 1
    afrho[i] = ( 40.0-2.5*log10(datacount/sky), error )

    #now start to count
    i += int(defaults[5])
    while i < r:
      phmat, rg = PhotomMatrix( y0, x0, i, image_data.shape )
      phmat -= phmat0 

      if i > int(defaults[6]):
        datacount += AverCircFlt( phmat, rg, image_data, sky, defaults )#matrix count
      else:
        datacount += np.sum( (image_data[ rg[0]:rg[1], rg[2]:rg[3] ] - sky) * phmat[ rg[0]:rg[1], rg[2]:rg[3] ] )
      #saturation protection
      if limit in ( image_data[ rg[0]:rg[1], rg[2]:rg[3] ] * phmat[ rg[0]:rg[1], rg[2]:rg[3] ]):
        error = 1
      afrho[i] = ( 40.0-2.5*log10(datacount/sky), error )
      
      phmat0 += phmat         
      i += int(defaults[5])

    phmat, rg = PhotomMatrix( y0, x0, r, image_data.shape )
    phmat -= phmat0
    datacount += AverCircFlt( phmat, rg, image_data, sky, defaults )#matrix count

    if limit in ( image_data[ rg[0]:rg[1], rg[2]:rg[3] ] * phmat[ rg[0]:rg[1], rg[2]:rg[3] ]):
      error = 1
    afrho[r] = ( 40.0-2.5*log10(datacount/sky), error )
    return 40.0-2.5*log10(datacount/sky), error, afrho         
