#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
# Settings
#"""

import datavar
import sys
#import koprdial
from PyQt5 import QtCore, QtWidgets, QtWidgets

font = datavar.DefaultFont()

# General function to write changes into file
def doWrite(fname, itemlist):
    f = open('./config-files/'+fname+'.dat', 'w')
    for item in itemlist:
       f.write( ','.join(item)+'\n' )
    f.close()

# Main dialog with settings tabs
class TabDialog(QtWidgets.QDialog):
    def __init__(self, observer, defaults, items, parent=None):
        super(TabDialog, self).__init__(parent)
        self.observer, self.defaults, self.items = observer, defaults, items

        tabWidget = QtWidgets.QTabWidget()

        self.loctab = SettingsTab(items['location'], [ 'Name', 'Longitude', 'Latitude', 'Altitude(m)', 'GMT + ' ])
        self.loctab.listWidget.itemClicked.connect(self.onItemloc)
        self.loctab.btnadd.clicked.connect(self.button_addloc)
        self.loctab.btnupd.clicked.connect(self.button_updloc)
        self.loctab.btnrem.clicked.connect(self.button_remloc)
        
        self.teltab = SettingsTab(items['telescope'], [ 'Name', 'Telescope type', 'Aperture diameter(mm)', 'Focl.(mm)', 'Zoom if fixed (optional)', 'FOV if fixed (optional)' ])
        self.teltab.listWidget.itemClicked.connect(self.onItemtel)
        self.teltab.btnadd.clicked.connect(self.button_addtel)
        self.teltab.btnupd.clicked.connect(self.button_updtel)
        self.teltab.btnrem.clicked.connect(self.button_remtel)

        try:
          if self.items['telescope'][0][3]: 
            self.FEnabler()
          elif not self.items['telescope'][0][3] and self.items['telescope'][0][4] and self.items['telescope'][0][5]:  
            self.FDisabler()
        except IndexError:
          pass

        datavar.clickable(self.teltab.elements[1]).connect(self.TelSelector)
        datavar.clickable(self.teltab.elements[3]).connect(self.FEnabler)
        datavar.clickable(self.teltab.elements[4]).connect(self.FDisabler)

        self.camtab = SettingsTab(items['camera'], [ 'Name', 'Camera ICQ code', 'Chip name', 'Chip ICQ code', 'Resolution X', 'Resolution Y', 'Pixel size X', 'Pixel size Y', 'Spectral sensitivity peak nm (Optional)' ])
        self.camtab.listWidget.itemClicked.connect(self.onItemcam)
        self.camtab.btnadd.clicked.connect(self.button_addcam)
        self.camtab.btnupd.clicked.connect(self.button_updcam)
        self.camtab.btnrem.clicked.connect(self.button_remcam)
        datavar.clickable(self.camtab.elements[1]).connect(self.CamSelector)
        datavar.clickable(self.camtab.elements[3]).connect(self.ChipSelector)
        
        self.eyetab = SettingsTab(items['eyepiece'], [ 'Name', 'Focal length(mm)', 'Apparent field' ])
        self.eyetab.listWidget.itemClicked.connect(self.onItemeye)
        self.eyetab.btnadd.clicked.connect(self.button_addeye)
        self.eyetab.btnupd.clicked.connect(self.button_updeye)
        self.eyetab.btnrem.clicked.connect(self.button_remeye)

        self.CCDDefaults = GeneralTab(defaults[3:], [ 'Afrho max radius (km)', 'Starting radius (px)', 'Radius steps (px)', 'ACF ignore radius (px)', 'ACF cut (% above average)', 'Max field of Vizier request (deg.)', 'Preferred corrections', 'Default filter', 'Calculated comet pos.', 'Show Vizier URL request' ]) # change

        self.GeneralTab = GeneralTab(observer, [ 'ICQ code', 'First name', 'Last name' ])
        tabWidget.addTab(self.GeneralTab, 'General')
        tabWidget.addTab(self.loctab, 'Locations')
        tabWidget.addTab(self.teltab, 'Telescopes')
        tabWidget.addTab(self.camtab, 'Cameras')
        tabWidget.addTab(self.eyetab, 'Eyepieces')
        tabWidget.addTab(self.CCDDefaults, 'CCD Defaults')
        tabWidget.setFont(font)

        buttonBox = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)

        buttonBox.accepted.connect(self.savequit)
        buttonBox.rejected.connect(self.reject)
        
        vbox = QtWidgets.QVBoxLayout()
        vbox.addWidget(tabWidget)
        vbox.addWidget(buttonBox)
        self.setLayout(vbox)
        self.setGeometry(200, 200, 600, 400)
        self.setWindowTitle('Settings')

    # Save after OK is clicked
    def savequit(self):
        observer = []
        defaults = self.defaults[:3]
        for item in self.GeneralTab.elements:
          observer.append(str(item.text()))
        for item in self.CCDDefaults.elements[:-4]:
          defaults.append(str(item.text()))
        defaults.append(str(self.CCDDefaults.elements[-4].currentIndex()))
        defaults.append(str(self.CCDDefaults.elements[-3].currentIndex()))
        defaults.append(str(self.CCDDefaults.elements[-2].checkState()))
        defaults.append(str(self.CCDDefaults.elements[-1].checkState()))
        if '' in observer:
          koprdial.InfoDialog(1, 'Input error', 'Fill all required fields!')
        else:
          self.observer = observer
          self.defaults = defaults
          self.close()

#### ************* LOCATIONS ************    
    #checking field input consistency
    def FieldCheckloc(self):
        check = True
        for i in range(len(self.loctab.elements)):
          if not self.loctab.elements[i].text():
            koprdial.InfoDialog(1, 'Input error', 'Fill all required fields!')
            check = False
            break
          elif i != 0 and not str(self.loctab.elements[i].text()).replace('-','').replace('.','').isdigit():
            koprdial.InfoDialog(1, 'Input error', 'Input is not digit!')
            check = False
            break
        return check
        
    #item selection
    def onItemloc(self):
        for i, item in enumerate(self.items['location'][self.loctab.listWidget.currentRow()]):
           self.loctab.elements[i].setText(item)

    #add location
    def button_addloc(self):
        self.items['location'].append(['new', '', '', '', ''])
        self.loctab.litem.append(QtWidgets.QListWidgetItem('new'))
        self.loctab.listWidget.addItem(self.loctab.litem[-1])
        for i, item in enumerate(self.items['location'][len(self.items['location'])-1]):
           self.loctab.elements[i].setText(item)
        self.loctab.listWidget.setCurrentRow(len(self.items['location'])-1)

    #update location
    def button_updloc(self):
        currow = self.loctab.listWidget.currentRow()
        if self.FieldCheckloc() and currow >= 0:
          for i in range(len(self.loctab.elements)):
            self.items['location'][self.loctab.listWidget.currentRow()][i] = str(self.loctab.elements[i].text())   
          doWrite('location', self.items['location'])
          self.loctab.listWidget.clear()
          self.loctab.litem = []
          for i, item in enumerate(self.items['location']):
            self.loctab.litem.append(QtWidgets.QListWidgetItem(item[0]))
            self.loctab.listWidget.addItem(self.loctab.litem[i])
          self.loctab.listWidget.setCurrentRow(currow)

    #remove locations
    def button_remloc(self):
        del self.items['location'][self.loctab.listWidget.currentRow()] 
        self.loctab.listWidget.clear()
        self.loctab.litem = []
        for i, item in enumerate(self.items['location']):
          self.loctab.litem.append(QtWidgets.QListWidgetItem(item[0]))
          self.loctab.listWidget.addItem(self.loctab.litem[i])

        doWrite('location', self.items['location'])

        if self.items['location']:
          self.loctab.listWidget.setCurrentRow(0)
          for i, item in enumerate(self.items['location'][self.loctab.listWidget.currentRow()]):
            self.loctab.elements[i].setText(item)

#### ************* TELESCOPES ************
    #checking field input consistency
    def FieldChecktel(self):
        check = True
        #check telescopes without fixed zoom
        # is text 0, 1, 2
        # normal is text 3
        # fixed zoom is text 4,5 
        for i in range(3):
          if not self.teltab.elements[i].text():
            koprdial.InfoDialog(1, 'Input error', 'Fill all required fields!')
            check = False
            break
          elif i > 1 and not str(self.teltab.elements[i].text()).replace('.','').isdigit():
            koprdial.InfoDialog(1, 'Input error', 'Input is not digit!')
            check = False
            break

        if not self.teltab.elements[3].text() and not self.teltab.elements[4].text() and not self.teltab.elements[5].text():
          koprdial.InfoDialog(1, 'Input error', 'Insert either Focal length for normal telescopes or Zoom and FOV for binoculars!')
          check = False

        elif self.teltab.elements[4].text() and not self.teltab.elements[5].text():
          koprdial.InfoDialog(1, 'Input error', 'For fixed zoom, you need to insert fixed FOV as well!')
          check = False

        elif self.teltab.elements[5].text() and not self.teltab.elements[4].text():
          koprdial.InfoDialog(1, 'Input error', 'For fixed FOV, you need to insert fixed Zoom as well!')
          check = False
          
        return check
        
    #item selection
    def onItemtel(self):
        for i, item in enumerate(self.items['telescope'][self.teltab.listWidget.currentRow()]):
           self.teltab.elements[i].setText(item)
        if self.items['telescope'][self.teltab.listWidget.currentRow()][3]: 
          self.FEnabler()
        elif not self.items['telescope'][self.teltab.listWidget.currentRow()][3] and self.items['telescope'][self.teltab.listWidget.currentRow()][4] and self.items['telescope'][self.teltab.listWidget.currentRow()][5]:  
          self.FDisabler()

    #add telation
    def button_addtel(self):
        self.FEnabler
        self.items['telescope'].append(['new', '', '', '','', ''])
        self.teltab.litem.append(QtWidgets.QListWidgetItem('new'))
        self.teltab.listWidget.addItem(self.teltab.litem[-1])
        for i, item in enumerate(self.items['telescope'][len(self.items['telescope'])-1]):
           self.teltab.elements[i].setText(item)
        self.teltab.listWidget.setCurrentRow(len(self.items['telescope'])-1)

    #update telation
    def button_updtel(self):
        currow = self.teltab.listWidget.currentRow()
        if self.FieldChecktel() and currow >= 0:
          for i in range(len(self.teltab.elements)):
            try:
              self.items['telescope'][self.teltab.listWidget.currentRow()][i] = str(self.teltab.elements[i].text())
            except IndexError:
              self.items['telescope'][self.teltab.listWidget.currentRow()].append(str(self.teltab.elements[i].text()))
                 
          doWrite('telescope', self.items['telescope'])
          self.teltab.listWidget.clear()
          self.teltab.litem = []
          for i, item in enumerate(self.items['telescope']):
            self.teltab.litem.append(QtWidgets.QListWidgetItem(item[0]))
            self.teltab.listWidget.addItem(self.teltab.litem[i])
          self.teltab.listWidget.setCurrentRow(currow)

    #remove telations
    def button_remtel(self):
        del self.items['telescope'][self.teltab.listWidget.currentRow()] 
        self.teltab.listWidget.clear()
        self.teltab.litem = []
        for i, item in enumerate(self.items['telescope']):
          self.teltab.litem.append(QtWidgets.QListWidgetItem(item[0]))
          self.teltab.listWidget.addItem(self.teltab.litem[i])

        doWrite('telescope', self.items['telescope'])
        
        if self.items['telescope']:
          self.teltab.listWidget.setCurrentRow(0)
          for i, item in enumerate(self.items['telescope'][self.teltab.listWidget.currentRow()]):
            self.teltab.elements[i].setText(item)

    def TelSelector(self):
        a = Selector( 'Telescope type', datavar.Telescopes(), str(self.teltab.elements[1].text()) )
        a.exec_()
        self.teltab.elements[1].setText(a.telcode)

    def FEnabler(self):
        self.teltab.elements[3].setDisabled(0)
        self.teltab.elements[4].setDisabled(1)
        self.teltab.elements[4].setText('')
        self.teltab.elements[5].setDisabled(1)        
        self.teltab.elements[5].setText('')

    def FDisabler(self):
        self.teltab.elements[3].setDisabled(1)
        self.teltab.elements[3].setText('')
        self.teltab.elements[4].setDisabled(0)
        self.teltab.elements[5].setDisabled(0)        
        
#### ************* CAMERAS ************
    #checking field input consistency
    def FieldCheckcam(self):
        check = True
        for i in range(len(self.camtab.elements)):
          if not self.camtab.elements[i].text() and i < 8:
            koprdial.InfoDialog(1, 'Input error', 'Fill all required fields!')
            check = False
            break
          elif i > 3 and i < 8 and not str(self.camtab.elements[i].text()).replace('.','').isdigit():
            koprdial.InfoDialog(1, 'Input error', 'Input is not digit!')
            check = False
            break
        return check
        
    def onItemcam(self):
        for i, item in enumerate(self.items['camera'][self.camtab.listWidget.currentRow()]):
           self.camtab.elements[i].setText(item)

    #add camation
    def button_addcam(self):
        self.items['camera'].append(['new', '', '', '', '', '', '', '', ''])
        self.camtab.litem.append(QtWidgets.QListWidgetItem('new'))
        self.camtab.listWidget.addItem(self.camtab.litem[-1])
        for i, item in enumerate(self.items['camera'][len(self.items['camera'])-1]):
           self.camtab.elements[i].setText(item)
        self.camtab.listWidget.setCurrentRow(len(self.items['camera'])-1)

    #update camation
    def button_updcam(self):
        currow = self.camtab.listWidget.currentRow()
        if self.FieldCheckcam() and currow >= 0:
          if len(self.items['camera'][currow]) == 8: # peak spectral sensitivity transition from older version
            self.items['camera'][currow].append('')
          for i in range(len(self.camtab.elements)):
            self.items['camera'][currow][i] = str(self.camtab.elements[i].text())   
          doWrite('camera', self.items['camera'])
          self.camtab.listWidget.clear()
          self.camtab.litem = []
          for i, item in enumerate(self.items['camera']):
            self.camtab.litem.append(QtWidgets.QListWidgetItem(item[0]))
            self.camtab.listWidget.addItem(self.camtab.litem[i])
          self.camtab.listWidget.setCurrentRow(currow)

    #remove camations
    def button_remcam(self):
        del self.items['camera'][self.camtab.listWidget.currentRow()] 
        self.camtab.listWidget.clear()
        self.camtab.litem = []
        for i, item in enumerate(self.items['camera']):
          self.camtab.litem.append(QtWidgets.QListWidgetItem(item[0]))
          self.camtab.listWidget.addItem(self.camtab.litem[i])

        doWrite('camera', self.items['camera'])
        
        if self.items['camera']:
          self.camtab.listWidget.setCurrentRow(0)
          for i, item in enumerate(self.items['camera'][self.camtab.listWidget.currentRow()]):
            self.camtab.elements[i].setText(item)

    def CamSelector(self):
        a = Selector( 'Camera name', datavar.Cameras(), str(self.camtab.elements[1].text()) )
        a.exec_()
        self.camtab.elements[1].setText(a.telcode)
        if not self.camtab.elements[0].text() or str(self.camtab.elements[0].text()) == 'new':
          self.camtab.elements[0].setText(a.telname)
        
    def ChipSelector(self):
        a = Selector( 'Camera Chip name', datavar.CCDChips(), str(self.camtab.elements[3].text()) )
        a.exec_()
        self.camtab.elements[3].setText(a.telcode)
        if not self.camtab.elements[2].text():
          self.camtab.elements[2].setText(a.telname)
        
#### ************* EYEPIECES ************
    #checking field input consistency
    def FieldCheckeye(self):
        check = True
        for i in range(len(self.eyetab.elements)):
          if not self.eyetab.elements[i].text():
            koprdial.InfoDialog(1, 'Input error', 'Fill all required fields!')
            check = False
            break
          elif i != 0 and not str(self.eyetab.elements[i].text()).replace('.','').isdigit():
            koprdial.InfoDialog(1, 'Input error', 'Input is not digit!')
            check = False
            break
        return check
        
    #item selection
    def onItemeye(self):
        for i, item in enumerate(self.items['eyepiece'][self.eyetab.listWidget.currentRow()]):
           self.eyetab.elements[i].setText(item)

    #add eyeation
    def button_addeye(self):
        self.items['eyepiece'].append(['new', '', ''])
        self.eyetab.litem.append(QtWidgets.QListWidgetItem('new'))
        self.eyetab.listWidget.addItem(self.eyetab.litem[-1])
        for i, item in enumerate(self.items['eyepiece'][len(self.items['eyepiece'])-1]):
           self.eyetab.elements[i].setText(item)
        self.eyetab.listWidget.setCurrentRow(len(self.items['eyepiece'])-1)

    #update eyeation
    def button_updeye(self):
        currow = self.eyetab.listWidget.currentRow()
        if self.FieldCheckeye() and currow >= 0:
          for i in range(len(self.eyetab.elements)):
            self.items['eyepiece'][self.eyetab.listWidget.currentRow()][i] = str(self.eyetab.elements[i].text())   
          doWrite('eyepiece', self.items['eyepiece'])
          self.eyetab.listWidget.clear()
          self.eyetab.litem = []
          for i, item in enumerate(self.items['eyepiece']):
            self.eyetab.litem.append(QtWidgets.QListWidgetItem(item[0]))
            self.eyetab.listWidget.addItem(self.eyetab.litem[i])
          self.eyetab.listWidget.setCurrentRow(currow)

    #remove eyeations
    def button_remeye(self):
        del self.items['eyepiece'][self.eyetab.listWidget.currentRow()] 
        self.eyetab.listWidget.clear()
        self.eyetab.litem = []
        for i, item in enumerate(self.items['eyepiece']):
          self.eyetab.litem.append(QtWidgets.QListWidgetItem(item[0]))
          self.eyetab.listWidget.addItem(self.eyetab.litem[i])
        
        doWrite('eyepiece', self.items['eyepiece'])
        
        if self.items['eyepiece']:
          self.eyetab.listWidget.setCurrentRow(0)
          for i, item in enumerate(self.items['eyepiece'][self.eyetab.listWidget.currentRow()]):
            self.eyetab.elements[i].setText(item)

#### ************* FUNCTIONS ************

#Tab with general settings
class GeneralTab(QtWidgets.QWidget):
    def __init__(self, observer, labels, parent=None):
        super(GeneralTab, self).__init__(parent)
        grid = QtWidgets.QGridLayout()

        for i in range(len(labels) - len(observer)):
          observer.append('0')
        
        self.elements = []
        for i, item in enumerate(labels):
          label = QtWidgets.QLabel(item)
          label.setFont(font)
          grid.addWidget(label, *(i,0))
          if i < 6:
            self.elements.append(QtWidgets.QLineEdit(self))
            self.elements[i].setFont(font)
            self.elements[i].setText(observer[i])
            grid.addWidget(self.elements[i], *(i,1))
          elif i == 6:
            self.elements.append(QtWidgets.QComboBox(self) )
            self.elements[i].setStyleSheet("selection-color: black")
            self.elements[i].setFont(font)
            for item in datavar.Corrections():
              self.elements[i].addItem(item[0])
            self.elements[i].setCurrentIndex(int(observer[6]))
            grid.addWidget(self.elements[i], *(i,1))
          elif i == 7:
            self.elements.append(QtWidgets.QComboBox(self) )
            self.elements[i].setStyleSheet("selection-color: black")
            self.elements[i].setFont(font)
            for item in datavar.Filters():
              self.elements[i].addItem(item[0])
            self.elements[i].setCurrentIndex(int(observer[7]))
            grid.addWidget(self.elements[i], *(i,1))
          else:
            self.elements.append(QtWidgets.QCheckBox(self) )
            self.elements[i].setFont(font)
            self.elements[i].setCheckState(int(observer[i]))
            grid.addWidget(self.elements[i], *(i,1))

        if len(labels) == 10: # change
          btndef = QtWidgets.QPushButton('Restore defaults', self)
          btndef.clicked.connect(self.button_restore)
          grid.addWidget(btndef, *(i+1,1))

        hbox = QtWidgets.QHBoxLayout()
        hbox.addLayout(grid)
        hbox.addStretch(1)

        vbox = QtWidgets.QVBoxLayout()
        vbox.addLayout(hbox)
        vbox.addStretch(1)
        self.setLayout(vbox)

    def button_restore(self):
        for i, item in enumerate([ '60000', '3', '2', '7', '100', '0', '3', '0', '0', '0' ]):
          if i < 6:
            self.elements[i].setText(item)
          elif i == 6:
            self.elements[i].setCurrentIndex(int(item))
          elif i == 7:
            self.elements[i].setCurrentIndex(int(item))
          else:
            self.elements[i].setCheckState(int(item))

#Tab with item lists        
class SettingsTab(QtWidgets.QWidget):
    def __init__(self, items, labels, parent=None):
        super(SettingsTab, self).__init__(parent)
        self.items = items
                
        hbox = QtWidgets.QHBoxLayout()
        self.listWidget, self.litem = QtWidgets.QListWidget(), []
        for i, item in enumerate(items):
          self.litem.append(QtWidgets.QListWidgetItem(item[0]))
          self.listWidget.addItem(self.litem[i])
        
        self.listWidget.setCurrentRow(0)
        
        grid = QtWidgets.QGridLayout()
        self.elements = []
        for i, item in enumerate(labels):
           label = QtWidgets.QLabel(item)
           label.setFont(font)
           grid.addWidget(label, *(i,0))
           self.elements.append(QtWidgets.QLineEdit(self))
           self.elements[i].setFont(font)
           if items:
             try:
               self.elements[i].setText(items[0][i])
             except IndexError:
               self.elements[i].setText('')
           grid.addWidget(self.elements[i], *(i,1))

        vbox = QtWidgets.QVBoxLayout()
        vbox.addLayout(grid)
        vbox.addStretch(1)

        hbox.addWidget(self.listWidget)
        hbox.addLayout(vbox)

        self.btnadd = QtWidgets.QPushButton('Add', self)
        self.btnupd = QtWidgets.QPushButton('Save', self)
        self.btnrem = QtWidgets.QPushButton('Remove', self)
        
        grid = QtWidgets.QGridLayout()
        for i, item in enumerate([self.btnadd, self.btnupd, self.btnrem]): 
          item.setFont(font)
          grid.addWidget(item, *(0,i))        

        vbox = QtWidgets.QVBoxLayout()
        vbox.addLayout(hbox)
        vbox.addLayout(grid)
        vbox.addStretch(1)
        self.setLayout(vbox)   
        
#Telescope types selection tab
class Selector(QtWidgets.QDialog):
    def __init__(self, text, telescopes, telcode, parent = None):
        super(Selector, self).__init__()

        self.telescopes = telescopes
        self.telcode = telcode  
        self.telname = ''

        hbox = QtWidgets.QHBoxLayout()
        label = QtWidgets.QLabel(text)
        label.setFont(font)
        self.code = QtWidgets.QLineEdit()
        self.code.setFont(font)
        hbox.addWidget(label)
        hbox.addWidget(self.code)

        self.output = QtWidgets.QComboBox()
        for item in list(self.telescopes.keys()):
          self.output.addItem(item)
        self.output.setFont(font)
        self.output.setStyleSheet("selection-color: black")
        self.output.activated.connect(self.onActivated)
        
        if telcode:
          self.code.setText(telcode)
          try:
            index = self.output.findText(list(self.telescopes.keys())[list(self.telescopes.values()).index(telcode)], QtCore.Qt.MatchFixedString)
          except ValueError:
            self.output.addItem('unknown')
            index = self.output.findText('unknown')
            if index >= 0:
              self.output.setCurrentIndex(index)
            

        vbox = QtWidgets.QVBoxLayout()
        vbox.addWidget(self.output)
        vbox.addLayout(hbox)

        btn = QtWidgets.QPushButton('Ok', self)
        btn.setFont(font)
        btn.clicked.connect(self.button_click)

        hbox = QtWidgets.QHBoxLayout()
        hbox.addStretch(1)
        hbox.addWidget(btn)

        vbox.addLayout(hbox)
        self.setLayout(vbox)  
        self.setWindowTitle(text+' selector')
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click(self):
        self.telcode = str(self.code.text())  
        self.telname = str(self.output.currentText())
        self.close()

    def onActivated(self):
        self.code.setText(self.telescopes[str(self.output.currentText())])                      
